// hub_mem

/*
-------------------------------------------------------------------------------
Copyright 2014 Parallax Inc.

This file is part of the hardware description for the Propeller 1 Design.

The Propeller 1 Design is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your option)
any later version.

The Propeller 1 Design is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
more details.

You should have received a copy of the GNU General Public License along with
the Propeller 1 Design.  If not, see <http://www.gnu.org/licenses/>.
-------------------------------------------------------------------------------
*/

// =======================================
// RR20140811 	Bigger Hub RAM & No ROM
//			    RAM is 48KB and remaps 48-64KB to 32-48KB
//				requires $readmemh("hub_rom_high.hex",memname,$8000,$BFFF);
// =======================================

module				hub_mem
(
input				clk_cog,
input				ena_bus,

input				w,
input		 [3:0]	wb,
input		[13:0]	a,
input		[31:0]	d,

output		[31:0]	q
);


// =======================================
// 16384 x 32 ram with byte-write enables ($0000..$FFFF) = 64KB for DE2
// 14336 x 32 ram with byte-write enables ($0000..$DFFF) = 56KB for DE0 (fails)
// 13312 x 32 ram with byte-write enables ($0000..$CFFF) = 52KB for DE0 (fails)
// 12800 x 32 ram with byte-write enables ($0000..$B7FF) = 50KB for DE0 (fails)
// 12288 x 32 ram with byte-write enables ($0000..$BFFF) = 48KB for DE0 (compiles)

reg [7:0] ram3 [12287:0];
reg [7:0] ram2 [12287:0];
reg [7:0] ram1 [12287:0];
reg [7:0] ram0 [12287:0];

// ??? $readmemh("hub_rom_high.hex",memname,$8000,$BFFF);

reg [7:0] ram_q3;
reg [7:0] ram_q2;
reg [7:0] ram_q1;
reg [7:0] ram_q0;

wire [13:0] addr = !a[13] ? a[13:0] : {2'b10, a[11:0]};	// remap RAM: 48-64KB overlays 32-48KB

always @(posedge clk_cog)
begin
	if (ena_bus && w && wb[3])
		ram3[addr[13:0]] <= d[31:24];
	if (ena_bus)
		ram_q3 <= ram3[addr[13:0]];
end

always @(posedge clk_cog)
begin
	if (ena_bus && w && wb[2])
		ram2[addr[13:0]] <= d[23:16];
	if (ena_bus)
		ram_q2 <= ram2[addr[13:0]];
end

always @(posedge clk_cog)
begin
	if (ena_bus && w && wb[1])
		ram1[addr[13:0]] <= d[15:8];
	if (ena_bus)
		ram_q1 <= ram1[addr[13:0]];
end

always @(posedge clk_cog)
begin
	if (ena_bus && w && wb[0])
		ram0[addr[13:0]] <= d[7:0];
	if (ena_bus)
		ram_q0 <= ram0[addr[13:0]];
end


assign q			= {ram_q3, ram_q2, ram_q1, ram_q0};

endmodule
